# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

import logging
import os
import sys
import bugzilla

from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_cors import CORS
from flask_oidc import OpenIDConnect
from flask_login import LoginManager
from celery import Celery

from . import config

# the version as used in setup.py
__version__ = "0.2.4"


def make_celery(app):
    c = Celery()
    # c.conf.update(app.config)

    class ContextTask(c.Task):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return self.run(*args, **kwargs)

    c.Task = ContextTask
    return c


class ReverseProxied(object):
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        scheme = environ.get('HTTP_X_FORWARDED_PROTO')
        if scheme:
            environ['wsgi.url_scheme'] = scheme
        return self.app(environ, start_response)

class TestingBugzilla():

    def getbug(self, bugid):
        return

    def url_to_query(self, url):
        return

    def query(self, query):
        return

# Flask App
app = Flask(__name__)
app.wsgi_app = ReverseProxied(app.wsgi_app)
app.secret_key = 'not-really-a-secret'

# Is this an OpenShift deployment?
openshift_production = os.getenv('OPENSHIFT_PROD')

default_config_dir = os.path.join(os.getcwd(), 'conf')
default_config_file = 'settings.py'

# Load default config, then override that with a config file
if os.getenv('DEV') == 'true':
    default_config_obj = 'oraculum.config.DevelopmentConfig'
elif os.getenv('TEST') == 'true':
    default_config_obj = 'oraculum.config.TestingConfig'
else:
    default_config_obj = 'oraculum.config.ProductionConfig'
    default_config_dir = '/etc/oraculum'

if 'celery' in sys.argv:
    default_config_obj = 'oraculum.config.CeleryWorkerConfig'
    default_config_file = 'celery_app_settings.py'


app.config.from_object(default_config_obj)

if openshift_production:
    config.openshift_config(app.config, openshift_production)

config_file = os.environ.get('ORACULUM_CONFIG', os.path.join(default_config_dir, default_config_file))

if os.path.exists(config_file):
    app.config.from_pyfile(os.path.realpath(config_file))

if app.config['PRODUCTION']:
    if app.secret_key == 'not-really-a-secret':
        raise Warning("You need to change the app.secret_key value for production")

# setup logging
fmt = '[%(filename)s:%(lineno)d] ' if app.debug else '%(module)-12s '
fmt += '%(asctime)s %(levelname)-7s %(message)s'
datefmt = '%Y-%m-%d %H:%M:%S'
loglevel = logging.DEBUG if app.debug else logging.INFO
formatter = logging.Formatter(fmt=fmt, datefmt=datefmt)


def setup_logging():
    root_logger = logging.getLogger('')
    root_logger.setLevel(logging.DEBUG)

    if app.config['STREAM_LOGGING']:
        print("doing stream logging")
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(loglevel)
        stream_handler.setFormatter(formatter)
        root_logger.addHandler(stream_handler)
        app.logger.addHandler(stream_handler)

    if app.config['SYSLOG_LOGGING']:
        print("doing syslog logging")
        syslog_handler = logging.handlers.SysLogHandler(
            address='/dev/log',
            facility=logging.handlers.SysLogHandler.LOG_LOCAL4)
        syslog_handler.setLevel(loglevel)
        syslog_handler.setFormatter(formatter)
        root_logger.addHandler(syslog_handler)
        app.logger.addHandler(syslog_handler)

    if app.config['FILE_LOGGING'] and app.config['LOGFILE']:
        print("doing file logging to %s" % app.config['LOGFILE'])
        file_handler = logging.handlers.RotatingFileHandler(
            app.config['LOGFILE'],
            maxBytes=500000,
            backupCount=5)
        file_handler.setLevel(loglevel)
        file_handler.setFormatter(formatter)
        root_logger.addHandler(file_handler)
        app.logger.addHandler(file_handler)


setup_logging()

if app.config['SHOW_DB_URI']:
    app.logger.debug('using DBURI: %s' % app.config['SQLALCHEMY_DATABASE_URI'])

app.config['JSON_AS_ASCII'] = False

# database
db = SQLAlchemy(app)

celery_app = make_celery(app)
celery_app.conf.update(
    broker_url=app.config['CELERY_BROKER_URL'],
    result_backend=app.config['CELERY_RESULT_BACKEND'],
    broker_transport_options={"queue_order_strategy":"priority"},
    worker_max_memory_per_child=524288,
)

from .utils import cache_utils  # NOQA: E402
CACHE = cache_utils.Cached(max_cache_age=app.config['MAX_DB_AGE'])

# let's not connect to the internet while in testing run
if not app.config["TESTING"]:
    BUGZILLA = bugzilla.Bugzilla("bugzilla.redhat.com",
                                tokenfile="/tmp/python-bugzilla-token-%d" % os.getpid(),
                                cookiefile="/tmp/python-bugzilla-cookie-%d" % os.getpid(),
                                force_rest=True)
else:
    BUGZILLA = TestingBugzilla()

CORS(app, supports_credentials=True)

oidc = OpenIDConnect(app)

login_manager = LoginManager()
login_manager.init_app(app)

from oraculum.controllers import cors_proxy  # NOQA: E402
from oraculum.controllers import kanban  # NOQA: E402
from oraculum.controllers import landing_page  # NOQA: E402
from oraculum.controllers import libkarma  # NOQA: E402
from oraculum.controllers import main  # NOQA: E402
from oraculum.controllers import oidc_login  # NOQA: E402
from oraculum.controllers import packager_dashboard  # NOQA: E402

application = app
