# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

import os
import sys
from argparse import ArgumentParser

from alembic.config import Config
from alembic import command as al_command
from alembic.migration import MigrationContext

from oraculum import db, CACHE
from oraculum.controllers.main import register_cache_providers
from oraculum.models.db_cache import CachedData


def get_alembic_config():
    # the location of the alembic ini file and alembic scripts changes when
    # installed via package
    if os.path.exists("./alembic.ini"):
        alembic_cfg = Config("./alembic.ini")
    else:
        alembic_cfg = Config("/usr/share/oraculum/alembic.ini",
                             ini_section='alembic-packaged')
    return alembic_cfg


def init_db():
    alembic_cfg = get_alembic_config()

    # check to see if the db has already been initialized by checking for an
    # alembic revision
    context = MigrationContext.configure(db.engine.connect())
    current_rev = context.get_current_revision()

    if current_rev:
        print("Database already initialized and at rev %s - not re-initializing" % current_rev)
        return True
    else:
        print("Initializing Database")
        db.drop_all()
        db.create_all()

        print("Initializing alembic version")

        al_command.stamp(alembic_cfg, "head")
        return True


def upgrade_db():
    print("Upgrading Database to Latest Revision")
    alembic_cfg = get_alembic_config()
    al_command.upgrade(alembic_cfg, "head")


def sync():
    print("Refreshing DB Cache")
    db.session.query(CachedData).delete()
    db.session.commit()

    register_cache_providers()
    for r in CACHE._refreshers:
        CACHE._refresh(r)

def debug():
    print("Launching oraculum in debug mode...")
    import oraculum
    oraculum.main.register_cache_providers()
    try:
        from IPython import embed
        from traitlets.config import get_config
    except ImportError:
        print("Make sure you have IPython installed for debug mode to work...")
        sys.exit(1)

    c = get_config()
    c.InteractiveShellEmbed.colors = "Linux"
    c.InteractiveShell.pdb = True
    embed(config=c)

def main():
    parser = ArgumentParser()
    parser.add_argument("command", choices=['init_db', 'upgrade_db', 'sync', 'debug'], help='Available commands')
    args = parser.parse_args()
    globals()[args.command]()


if __name__ == '__main__':
    main()
