# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

import os
import sys
import tempfile

from collections import defaultdict


class Config(object):
    DEBUG = True
    SQLALCHEMY_DATABASE_URI = 'sqlite://'

    LOGFILE = '/var/log/oraculum/oraculum.log'
    FILE_LOGGING = False
    SYSLOG_LOGGING = False
    STREAM_LOGGING = True

    RUN_HOST = None
    RUN_PORT = None

    PRODUCTION = False
    TESTING = False

    SHOW_DB_URI = False

    OIDC_CLIENT_SECRETS = './conf/client_secrets.json.example'
    OIDC_ID_TOKEN_COOKIE_SECURE = False
    OIDC_REQUIRE_VERIFIED_EMAIL = False
    OIDC_SCOPES = ['openid', 'email', 'profile']

    CELERY_BROKER_URL = 'redis://localhost:6379'
    CELERY_RESULT_BACKEND = 'redis://localhost:6379'

    ENABLE_LOCAL_CACHE = True

    # IGNORE_CACHE_TIMESTAMP sets how oraculum verifies data aga in cache
    # values can be 'local', 'db' or None
    # "local" - always trust both local and database caches, plan refresh if database data are missing
    # "db" - verify only local cache age, always trust database cache, plan refresh if database data are missing
    # None - verify age of both, local and database caches, plan refresh if data are too old
    IGNORE_CACHE_TIMESTAMP = 'db'

    MAX_DB_AGE = 14400  # keep data cached for 240 minutes
    ACTIVITY_REQUIRED = 14  # cache users who used the service in at least last 14 days

    PACKAGE_MAINTAINERS_JSON_URL = 'https://src.fedoraproject.org/extras/pagure_owner_alias.json'
    ORPHANS_JSON_URL = 'https://churchyard.fedorapeople.org/orphans.json'
    KOSCHEI_API_URL = 'https://koschei.fedoraproject.org/api/v1/packages'
    HEALTH_CHECK_URL = 'https://repochecker.decathorpe.com/data/'
    KOJI_URL = 'https://koji.fedoraproject.org/koji/'
    KOJIHUB_URL = 'https://koji.fedoraproject.org/kojihub/'
    BLOCKERBUGS_URL = 'https://qa.fedoraproject.org/blockerbugs/'
    SCHEDULE_URL = 'https://fedorapeople.org/groups/schedule/f-%d/f-%d-key.ics'

    EPEL_RELEASES = [6, 7, 8]

    # How often should be watchdog triggered
    # (For sending emails with health check report and counter resetting if ACTIVE_WATCHDOG)
    WATCHDOG_INTERVAL = 86400

    # ACTIVE_WATCHDOG handles counter reset to make oraculum even more robust in production deployment
    # Allows scheduling tasks again in cases when redis failed/crashed during runtime
    ACTIVE_WATCHDOG = True

    # Specify groups that are allowed to own packages, regexes and explicit names are allowed
    ALLOWED_PACKAGER_GROUPS = ["^sssd-maintainers$", ".+[-_]sig$"]

    SYNC_INTERVALS = defaultdict(lambda: 3600)
    # You can set specific tasks' sync intervals here, unset tasks will be refreshed in the
    #  default interval above
    # Available syncers:
    #    blocker_trackers
    #    bodhi_overrides
    #    bodhi_updates
    #    easyfix
    #    fedora_easy_karma
    #    fedora_manual_testing
    #    fedora_releases
    #    ftbfs_trackers
    #    fti_trackers
    #    health_check_data
    #    heart_beat
    #    koschei_data
    #    landing_page
    #    orphans_json
    #    packager_dashboard_static_user_data
    #    packager_dashboard_package_bugs
    #    packager_dashboard_package_prs
    #    packager_dashboard_package_versions
    #    packages_owners_json
    #    pagure_groups
    SYNC_INTERVALS['heart_beat'] = 10
    SYNC_INTERVALS["pagure_groups"] = 7200
    SYNC_INTERVALS["packager_dashboard_static_user_data"] = 3600
    SYNC_INTERVALS["packager_dashboard_package_bugs"] = 7200
    SYNC_INTERVALS["packager_dashboard_package_prs"] = 7200
    SYNC_INTERVALS["packager_dashboard_package_versions"] = 43200

    # Specify time (seconds) after which are tasks considered to be in "rotting state"
    TASKS_ROT_AFTER = 7200

    # FAILURE EMAILS
    # To receive failres via email, you need to set up smtp connection details and set SEND_ERROR_EMAILS to True
    SEND_ERROR_EMAILS = False
    SMTP_SERVER = ""
    SMTP_PORT = 465
    SMTP_LOGIN = ""
    SMTP_SENDER = ""
    SMTP_PASSWORD = ""

    # List of email addresses to send failures to
    ADMIN_EMAILS = []


class ProductionConfig(Config):
    PRODUCTION = True
    DEBUG = False
    FORCE_CACHED_DATA = True
    OIDC_CLIENT_SECRETS = '/etc/oraculum/client_secrets.json'


class DevelopmentConfig(Config):
    TRAP_BAD_REQUEST_ERRORS = True
    SQLALCHEMY_DATABASE_URI = 'sqlite:////tmp/oraculum_db.sqlite'

    RUN_HOST = '0.0.0.0'
    RUN_PORT = 5000


class TestingConfig(Config):
    TRAP_BAD_REQUEST_ERRORS = True
    TESTING = True


class CeleryWorkerConfig(Config):
    ENABLE_LOCAL_CACHE = False


def openshift_config(config_object, openshift_production):
    # First, get db details from env
    try:
        config_object["SQLALCHEMY_DATABASE_URI"] = "postgresql+psycopg2://%s:%s@%s:%s/%s" % (
            os.environ["POSTGRESQL_USER"],
            os.environ["POSTGRESQL_PASSWORD"],
            os.environ["POSTGRESQL_SERVICE_HOST"],
            os.environ["POSTGRESQL_SERVICE_PORT"],
            os.environ["POSTGRESQL_DATABASE"]
        )
        config_object["SECRET_KEY"] = os.environ["SECRET_KEY"]

        # Creates a temporary file containing the CLIENT_SECRETS data
        #  and sets OIDC_CLIENT_SECRETS to the files path
        temp = tempfile.NamedTemporaryFile(mode="w+", delete=False)
        temp.write(os.environ["CLIENT_SECRETS"])
        temp.close()
        config_object["OIDC_CLIENT_SECRETS"] = temp.name
    except KeyError:
        print("OpenShift mode enabled but required values couldn't be fetched. "
              "Check, if you have these variables defined in you env: "
              "(POSTGRESQL_[USER, PASSWORD, DATABASE, SERVICE_HOST, SERVICE_PORT], SECRET_KEY, CLIENT_SECRETS)",
              file=sys.stderr)
        sys.exit(1)

    # Let's try redis
    try:
        config_object["CELERY_BROKER_URL"] = "redis://:%s@%s:%s" % (
            os.environ["REDIS_PASSWORD"],
            os.environ["REDIS_SERVICE_HOST"],
            os.environ["REDIS_SERVICE_PORT_REDIS"]
        )
        config_object["CELERY_RESULT_BACKEND"] = config_object["CELERY_BROKER_URL"]
    except KeyError:
        print("OpenShift mode enabled but Redis instance couldn't be reached."
              "Please, make sure REDIS_[PASSWORD, SERVICE_HOST, SERVICE_PORT_REDIS] are available in env.",
              file=sys.stderr)
        sys.exit(1)

    # Get some more data from OpenShift, if set
    if os.getenv("KANBAN_CONFIG"):
        config_object["KANBAN_CONFIG"] = os.getenv("KANBAN_CONFIG")
    if os.getenv("ALLOWED_PACKAGER_GROUPS"):
        config_object["ALLOWED_PACKAGER_GROUPS"] = os.getenv("ALLOWED_PACKAGER_GROUPS")
    if os.getenv("EPEL_RELEASES"):
        config_object["EPEL_RELEASES"] = os.getenv("EPEL_RELEASES")
    if os.getenv("SMTP_CONFIG"):
        smtp_config = os.getenv("SMTP_CONFIG").split(";")
        config_object["SEND_ERROR_EMAILS"] = True
        config_object["SMTP_SERVER"] = smtp_config[0]
        config_object["SMTP_PORT"] = smtp_config[1]
        config_object["SMTP_LOGIN"] = smtp_config[2]
        config_object["SMTP_SENDER"] = smtp_config[3]
        config_object["SMTP_PASSWORD"] = smtp_config[4]
        config_object["ADMIN_EMAILS"] = os.getenv("ADMIN_EMAILS")

    # Some final touches for oidc
    if os.getenv("OVERWRITE_REDIRECT_URI"):
        config_object["OVERWRITE_REDIRECT_URI"] = os.getenv("OVERWRITE_REDIRECT_URI")
