#
# bodhi.py - Bodhi parsers and utilities for packager dashboard
#
# Copyright 2020, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Frantisek Zatloukal <fzatlouk@redhat.com>
#   Josef Skladanka <jskladan@redhat.com>

from bodhi.client.bindings import BodhiClient

import datetime

from oraculum import app
from oraculum.utils.dashboard_helpers import release_from_nvr, name_in_nevra


def query_bodhi(bodhi_client, release, pending=False):
    """Deal with querying bodhi and combining all relevant pages into a
    single list of updates."""
    query_args = {"release": str(release),
                  "rows_per_page": 50,
                  }
    if pending:
        query_args["request"] = "testing"
        query_args["status"] = "pending"
    else:
        query_args["status"] = "testing"

    updates = []
    try:
        # since bodhi has a query limit but multiple pages, get ALL of the
        # updates before starting to process
        result = bodhi_client.query(**query_args)
        updates.extend(result['updates'])
        while result.page < result.pages:
            next_page = result['page'] + 1
            app.logger.debug("Fetching updates page {} of {}".format(
                next_page, result['pages']))
            result = bodhi_client.query(page=next_page, **query_args)
            updates.extend(result['updates'])
    # There is no clear indication which Exceptions bc.query() might
    # throw, therefore catch all (python-fedora-0.3.32.3-1.fc19)
    except Exception as e:
        app.logger.debug("Error while querying Bodhi: {0}".format(e))

    return updates


def process_update(update):
    """
    Cleans up single update dictionary to contain only data frontend needs
    """
    return {
        "pretty_name": update["title"],
        "updateid": update["alias"],
        "submission_date": update["date_submitted"],
        "stable_by_time": str(
                datetime.datetime.fromisoformat(update["date_modified"] or update["date_submitted"]) +
                datetime.timedelta(days=update["stable_days"])
            ) if update["autotime"] else None,
        "release": release_from_nvr(update["title"]),
        "url": update["url"],
        "status": "%s→%s" % (update["status"], update["request"]) if update["request"] else update["status"],
        "karma": update["karma"],
        "comments": len(update["comments"])
    }


def process_override(override):
    """
    Cleans up single override dictionary to contain only data frontend needs
    """
    return {
        "pretty_name": override["nvr"],
        "url": "https://bodhi.fedoraproject.org/overrides/" + override["nvr"],
        "submission_date": override["submission_date"],
        "expiration_date": override["expiration_date"],
        "release": release_from_nvr(override["nvr"])
    }


def get_updates(packages, raw_updates):
    """
    Gets list of user owned packages and libkarma dump of all updates
    Returns dict of user owned packages and updates for them
    package_name: {
        "pretty_name": update["title"],
        "updateid": update["alias"],
        "submission_date": update["date_submitted"],
        "stable_by_time": (update["date_submitted"] + update["stable_days"]) if update["autotime"] else None,
        "release": release_from_nvr(update["title"]),
        "url": update["url"],
        "status": update["status"],
        "karma": update["karma"],
        "comments": len(update["comments"])
    }
    """
    if len(packages) == 0:
        return {}
    data = {}
    # Prepare dict items for each package
    for package in packages:
        data[package] = []

    # raw_updates["Fxx"][0...n]["builds"][0...n]["nvr"] = 'package_name-version-release'
    # raw_updates["Fxx"][0...n]["alias"] = FEDORA-YYYY-UPDATE_ID
    for release in raw_updates:
        for package in packages:
            present_updates = set()  # helper set to deduplicate updates containing more than one package
            for update in raw_updates[release]:
                for build in update["builds"]:
                    if name_in_nevra(package, build["nvr"]) and update["alias"] not in present_updates:
                        data[package].append(process_update(update))
                        present_updates.add(update["alias"])

    return data


def get_overrides():
    """
    Returns list of all active buildroot overrides
    """
    bc = BodhiClient(username="oraculum",
                     useragent="Fedora Easy Karma/GIT",
                     retries=3)

    query_args = {"expired": False,
                  "rows_per_page": 50,
                  }

    overrides = []
    try:
        # since bodhi has a query limit but multiple pages, get ALL of the
        # updates before starting to process
        result = bc.list_overrides(**query_args)
        overrides.extend(result['overrides'])
        while result.page < result.pages:
            next_page = result['page'] + 1
            app.logger.debug("Fetching overrides page {} of {}".format(
                next_page, result['pages']))
            result = bc.list_overrides(page=next_page, **query_args)
            overrides.extend(result['overrides'])
    # There is no clear indication which Exceptions bc.query() might
    # throw, therefore catch all (python-fedora-0.3.32.3-1.fc19)
    except Exception as e:
        app.logger.error("Error while querying Bodhi: {0}".format(e))

    return overrides


def get_user_overrides(packages, raw_overrides):
    """
    Returns all active buildroot overrides for single user
    Formatted as dict of dicts:
    package_name: {
        "pretty_name": override["nvr"],
        "url": "https://bodhi.fedoraproject.org/overrides/" + override["nvr"],
        "submission_date": override["submission_date"],
        "expiration_date": override["expiration_date"],
        "release": release_from_nvr(override["nvr"])
    }
    """
    data = {}
    for package in packages:
        data[package] = []
    for override in raw_overrides:
        for package in packages:
            if name_in_nevra(package, override["nvr"]):
                data[package].append(process_override(override))
    return data


def get_updates_from_bodhi(release):
    bc = BodhiClient(username="oraculum",
                     useragent="Fedora Easy Karma/GIT",
                     retries=3)

    updates = query_bodhi(bc, release, False)
    updates_pending = query_bodhi(bc, release, True)
    updates.extend(updates_pending)

    return updates
