#
# bugzilla.py - Bugzilla parsers and utilities for packager dashboard
#
# Copyright 2020, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Frantisek Zatloukal <fzatlouk@redhat.com>
#   Josef Skladanka <jskladan@redhat.com>

import urllib.parse

from oraculum import CACHE, BUGZILLA
from oraculum.utils.dashboard_helpers import release_from_number


def get_blocker_trackers():
    tracker_ids = []
    releases = CACHE.get('fedora_releases')
    if releases["branched"]:
        try:
            tracker_ids.append(BUGZILLA.getbug("F%sBetaBlocker" % releases["branched"]).id)
            tracker_ids.append(BUGZILLA.getbug("F%sFinalBlocker" % releases["branched"]).id)
        except IndexError:
            pass
    try:
        tracker_ids.append(BUGZILLA.getbug("F%sBetaBlocker" % releases["rawhide"]).id)
        tracker_ids.append(BUGZILLA.getbug("F%sFinalBlocker" % releases["rawhide"]).id)
    except IndexError:
        pass
    return tracker_ids


def get_fti_trackers():
    tracker_ids = []
    releases = CACHE.get('fedora_releases')["values"]
    for release in releases:
        try:
            tracker_ids.append(BUGZILLA.getbug("F%sFailsToInstall" % release).id)
        except IndexError:
            continue
    return tracker_ids


def get_ftbfs_trackers():
    tracker_ids = []
    releases = CACHE.get('fedora_releases')["values"]
    for release in releases:
        try:
            tracker_ids.append(BUGZILLA.getbug("F%sFTBFS" % release).id)
        except IndexError:
            continue
    return tracker_ids


def get_package_bugs(package):
    """
    Returns all open Bugs for a single package
    """
    data = []
    query = BUGZILLA.url_to_query("https://bugzilla.redhat.com/buglist.cgi?bug_status=NEW&bug_status=__open__&"
                                  "classification=Fedora&product=Fedora&product=Fedora EPEL&component=%s" % urllib.parse.quote(package))
    query["include_fields"] = ['blocks', 'comments', 'creation_time', 'creator', 'id', 'keywords', 'last_change_time',
                               'severity', 'priority', 'status', 'summary', 'version', 'whiteboard']
    bugs = BUGZILLA.query(query)
    if len(bugs) == 0:
        return []
    ftbfs_trackers = set(CACHE.get('ftbfs_trackers'))
    fti_trackers = set(CACHE.get('fti_trackers'))
    blocker_trackers = set(CACHE.get('blocker_trackers'))
    for bug in bugs:
        if bug.creator == 'Upstream Release Monitoring':
            bug.keywords.append("ReleaseMonitoring")
        for blocks in bug.blocks:
            if blocks in ftbfs_trackers and "FTBFS" not in bug.keywords:
                bug.keywords.append("FTBFS")
            if blocks in fti_trackers and "FTI" not in bug.keywords:
                bug.keywords.append("FTI")
            if blocks in blocker_trackers and "AcceptedBlocker" in bug.whiteboard and "AcceptedBlocker" not in bug.keywords:
                bug.keywords.append("AcceptedBlocker")
            if blocks in blocker_trackers and "AcceptedBlocker" not in bug.whiteboard and "ProposedBlocker" not in bug.keywords:
                bug.keywords.append("ProposedBlocker")
        strs = ['unspecified', 'low', 'medium', 'high', 'urgent']
        release = release_from_number(bug.version.replace("epel", "").replace("el", ""))
        if "[fedora-all]" in bug.summary:
            release = "Fedora"
        if "[epel-all]" in bug.summary:
            release = "EPEL"
        data.append({
            "title": bug.summary,
            "bug_id": bug.id,
            "severity": bug.severity,
            "priority": bug.priority,
            "priority_severity": strs[max(strs.index(bug.priority), strs.index(bug.severity))],
            "status": bug.status,
            # moment.js on frontend requires RFC2822 datetime format, bugzilla returns RFC339
            "modified": str(bug.last_change_time).replace("T", " ").replace("Z", ""),
            "reported": str(bug.creation_time).replace("T", " ").replace("Z", ""),
            "release": release,
            "keywords": bug.keywords,
            "comments": (len(bug.comments) - 1),  # Let's ignore the first comment that every bug has
            "url": "https://bugzilla.redhat.com/%s" % bug.id
        })
    return data
