#
# koschei.py - Koschei parsers and utilities for packager dashboard
#
# Copyright 2020, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Frantisek Zatloukal <fzatlouk@redhat.com>
#   Josef Skladanka <jskladan@redhat.com>

from collections import defaultdict

import koji

from oraculum import app, CACHE
from oraculum.utils.dashboard_helpers import get_json, release_from_dist, release_is_active


def process_koji_queue(session, data):
    koji_results = session.multiCall()
    for element in koji_results:
        try:
            # koji multicall returns results packed inside lists of one element, redefine element for better readability later
            element = element[0][0]
        except IndexError:
            # Sometimes, koji returns incomplete results
            continue
        for entry in data[element["name"]]:
            """
            We need to iterate throughout all members of data["package_name"] to find and alter only the architecture we
            now have in the element.
            Example of element (only keys we use are listed below):
               {'tag_name': 'fXX',
                'build_id': XYZ,
                'completion_time': 'YYYY-MM-DD HH:MM:SS.FFFFFF',
                'name': 'package_name',
                ...}
            """
            # Since dict doesn't guarantee order, we must pair koji result and data entry
            if release_from_dist(element["tag_name"]) == entry["release"]:
                data[element["name"]][data[element["name"]].index(entry)]["last_success"] = {
                    "time": element["completion_time"],
                    "url": app.config["KOJI_URL"] + "buildinfo?buildID=" + str(element["build_id"])}
                # And we can jump to another element early if we found what we needed, yay!
                continue
    return data


def parse_koschei_data():
    """
    Prepares data from koschei for easier parsing in get_user_koschei_data()
    Returns dict of lists containing dicts: {
        "package_name": [
            {"release": "fedora_release", "status": "koschei status", "url": "https://koschei.fedoraproject.org/package/..."}
        ]
    }
    """
    data = defaultdict(list)
    koschei_resp = get_json(app.config['KOSCHEI_API_URL'])
    # Set up koji API
    koji_session = koji.ClientSession(app.config['KOJIHUB_URL'])
    koji_session.multicall = True

    for item in koschei_resp:
        if "playground" in item["collection"]:
            # We don't care about EPEL 8 Playground
            continue
        if not release_is_active(release_from_dist(item["collection"])):
            # koschei might contain data for EOL Fedora releases, we don't care about those
            continue
        data[item["name"]].append({
            "release": release_from_dist(item["collection"]),
            "status": item["state"],
            "url": "https://koschei.fedoraproject.org/package/%s?collection=%s" % (item["name"], item["collection"]),
            "last_success": {"time": None, "url": None}})
        if item["state"] == "failing":
            koji_session.getLatestBuilds(item["collection"], package=item["name"])

    return process_koji_queue(koji_session, data)


def get_user_koschei_data(packages):
    """
    Filters out koschei data to contain information only about given pacakges list
    """
    koschei = CACHE.get("koschei_data")
    data = {}
    for package in packages:
        data[package] = koschei.get(package, [])
    return data
