#
# orphans.py - Orphans parsers and utilities for packager dashboard
#
# Copyright 2020, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Frantisek Zatloukal <fzatlouk@redhat.com>
#   Josef Skladanka <jskladan@redhat.com>

import igraph

from oraculum import CACHE


class OrphanGraph(object):

    def __init__(self, orphans_data):
        self.orphans_data = orphans_data
        aff = orphans_data['affected_packages']
        edges = []
        self.pimap = {}
        self.ipmap = {}
        for i, p in enumerate(aff.keys()):
            self.pimap[p] = i
            self.ipmap[i] = p
        self.ileafs = [self.pimap[p] for p in orphans_data['status_change'].keys()]

        for p in aff:
            for d in aff[p]:
                edges.append((self.pimap[p], self.pimap[d]))

        self.g = igraph.Graph(directed=True)
        self.g.add_vertices(len(self.pimap)+1)
        self.g.add_edges(edges)

    def get_package_info(self, package):
        p = package

        if p not in self.orphans_data["affected_packages"]:
            return {
                "orphaned": False,
                "depends_on_orphaned": False,
                "direct_dependencies": [],
                "remote_dependencies": [],
                "problematic_since": None,
                "dot_graph": "",
                "vis_js": {"nodes": [], "edges": []}
            }

        paths = [_ for _ in self.g.get_shortest_paths(self.pimap[p], self.ileafs) if len(_) >= 2]
        direct_deps = list(set([self.ipmap[l[-1]] for l in paths if len(l) == 2]))
        remote_deps = list(set([self.ipmap[l[-1]] for l in paths if len(l) > 2]))

        try:
            # the "Z"*20 hack is here to 'fool' the min operator into ignoring the package in question, when it's not in "status_change"
            problematic_since = min([self.orphans_data["status_change"].get(d, "Z" * 20) for d in direct_deps + remote_deps + [p]])
        except ValueError:
            problematic_since = None
        if problematic_since == "Z" * 20:
            problematic_since = None

        dot_graph = ""
        for path in paths:
            dot_graph += ' -- '.join('"%s"' % self.ipmap[i] for i in path) + ";\n"

        _nodes = set()
        _edges = set()
        for path in paths:
            _nodes.update(path)
            for i in range(len(path)-1):
                _edges.add(tuple(path[i:i+2]))

        nodes = [{'id': n, 'label': self.ipmap[n]} for n in _nodes]
        edges = [{'from': e[0], 'to': e[1]} for e in _edges]

        return {
            "orphaned": p in self.orphans_data["status_change"].keys(),
            "depends_on_orphaned": bool(len(self.orphans_data["affected_packages"][p])),
            "direct_dependencies": direct_deps,
            "remote_dependencies": remote_deps,
            "problematic_since": problematic_since,
            "dot_graph": dot_graph.strip(),
            "vis_js": {'nodes': nodes, 'edges': edges},
        }


def get_orphans(packages, orphans_data=None):

    if not orphans_data:
        orphans_data = CACHE.get('orphans_json')

    graph = OrphanGraph(orphans_data)
    orphans = {}
    for package in packages:
        orphans[package] = graph.get_package_info(package)
    return orphans
