#
# schedule.py - Fedora Release Schedule parsers and utilities for QA Ladning Page
#
# Copyright 2020, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Frantisek Zatloukal <fzatlouk@redhat.com>
#   Josef Skladanka <jskladan@redhat.com>
#   Lukas Brabec <lbrabec@redhat.com>

import icalendar
import datetime
import pytz

from oraculum import app
from oraculum.utils.dashboard_helpers import get_resource

def _is_relevant(summary):
    relevant = ['Branch Fedora', 'Beta Freeze', 'Beta Release Public Availability', 'Final Freeze', 'Final Release Public Availability']
    return any([word in summary for word in relevant])


def get_schedule():

    # Work around cyclic import
    from oraculum import CACHE

    today = datetime.datetime.utcnow()
    today = today.replace(tzinfo=pytz.UTC)

    releases = CACHE.get("fedora_releases")
    release = releases["branched"] or releases["rawhide"]

    resp = get_resource(app.config["SCHEDULE_URL"] % (release, release))
    data = []

    if not resp:
        return []

    ical_raw = icalendar.Calendar.from_ical(resp)
    for c in ical_raw.walk():
        if c.name == "VEVENT":
            summary = c.get('summary')
            date = c.get('dtstart').dt.astimezone(pytz.UTC)
            if _is_relevant(summary):
                data.append({
                    'summary': summary,
                    'date': date.strftime("%d %b %Y"),
                    'dtdate': date,
                    'current': date <= today  # mark all until now as current and clean up later
                })

    data = sorted(data, key=lambda c: c['dtdate'])

    # leave the most recent 'current' milestone and clean up rest
    found = False
    for d in data[::-1]:
        if d['current'] and not found:
            found = True
            continue
        d['current'] = False

    # FIXME handle this better
    prebranch = {
        'summary': 'Rawhide',
        'date': today.strftime("%d %b %Y"),
        'dtdate': today,
        'current': not any([d['current'] for d in data]),
    }
    data = [prebranch] + data
    data_cleaned = []
    for schedule_event in data:
        data_cleaned.append({
            'summary': schedule_event["summary"],
            'date': schedule_event["date"],
            'current': schedule_event["current"]
        })

    return data_cleaned
